;===================================================================================================================== EQUATES

; INSTRUMENT EQUATES

MOD_INST_NAME	EQU		0		; Name
MOD_INST_LEN	EQU		22		; Length in words
MOD_INST_FINE	EQU		24		; Fine tune (low nibble signed)
MOD_INST_VOL	EQU		25		; Volume
MOD_INST_ROFF	EQU		26		; Repeat offset
MOD_INST_RLEN	EQU		28		; Repeat length in words
		
;--------------------------------------------------------------------------------------------------------------------- modinit
; Initialise our data

modinit:
		.EXTERN	module_ptr	
		.EXTERN song_ptr
		.EXTERN pattern_ptr
		.EXTERN maxposition_dsp
		
		move.l	moduleaddy,a0	; Actual module data location
		
		move.l	a0,module_ptr	; set the pointer to the module for the player
		add.l	#952,a0
		
		move.l	a0,song_ptr		; save the pointer to where the song data is
		add.l	#132,a0
		
		move.l	a0,pattern_ptr	; save the pointer to where the pattern data is
		
		move.l	moduleaddy,a0
		
		; need to find out where the samples start
		move.l	#$3b8,d0	; offset to patten data
		clr.l	d2
		move.w	#127,d7		; 128 positions to check
		move.l	#950,d1		; Position for number of song positions
		move.b	(a0,d1),d2
		
		and.l	#$ffff,d2
		
		move.l	d2,maxposition_dsp  ; store the maximum number of song positions (dsp)
		clr.l	d1		
.ploop:

		move.b	(a0,d0),d2
		
		cmp.b	d1,d2		; Do we have a higher pattern number?
		ble.s	.nochange
		move.b	d2,d1		; Yes, save the new top pattern number
.nochange:
		add.l	#1,d0
		dbra.w	d7,.ploop
		
		add.b	#1,d1
		and.w	#$ff,d1			; ensure its a clean word
		
		move.l	a0,a1
		
								; d1 now contains the number of patterns in this module
		lsl.l	#8,d1			; multiply by 1024  ( <<10  )
		lsl.l	#2,d1
		add.l	#$43c,d1		; Add on the start offset for pattern data
								; d1 now holds the offset for the 1st sample
							
		add.l	d1,a0			; Base address of instrument sample data in A0

		add.l	#20,a1			; the offset to the instrument data fields in A1
		
								; Create our own instrument data records for simpler playback
		.EXTERN	dsp_sample_tbl						
		lea		dsp_sample_tbl,a3	; populate the DSP sample table too
		move.w	#30,d7			; Upto 31 instruments
.iloop:

		move.l	a0,(a3)			; save length to DSP
		clr.l	d0
		clr.l	d5				; d5 for DSP
		
		move.w	MOD_INST_LEN(a1),d0		; get the length of the sample
		
		lsl.l	#1,d0			; convert to bytes
		
		move.l	a0,d5
		add.l	d0,a0			; update A0 to point to the next sample
		move.l	a0,4(a3)		; store the sample END address in DSP table
		
		clr.l	d0
		move.w	MOD_INST_ROFF(a1),d0	; get the repeat offset
		
		lsl.l	#1,d0				; words to bytes
		add.l	d5,d0
		move.l	d0,8(a3)			; store ROFF in DSP table
		move.l	d0,d5
		
		clr.l	d0
		move.w	MOD_INST_RLEN(a1),d0	; get the repeat end
		
		
		cmpi.w	#1,d0
		ble.s	.noloop
		lsl.l	#1,d0				; words to bytes
		addq	#1,d0				; test removes one, so add one here 
		add.l	d5,d0
		move.l	d0,12(a3)			; store RLEN in DSP table
.noloop:		
		clr.l	d0
		move.b	MOD_INST_VOL(a1),d0
		
		move.l	d0,16(a3)			; store (Null word |FINE | VOL) in DSP table
		
		move.l	#1990,20(a3)		; set default playback rate  *FIXME* TESTING VALUE
		
.nexti:
		add.l	#30,a1		; point to next instrument data field
		
		add.l	#24,a3		; next position in DSP sample table 
		dbra.w	d7,.iloop
				
		rts

;======================================================================================================================== DATA
.EXTERN buffers
buffers:	dc.w	1		; buffer counter

;====================================================================================================================== MODULE
.EVEN

moduleaddy:	dc.l	module





